import { FormBuilder, FormControl, FormsModule, ReactiveFormsModule, Validators } from '@angular/forms';
import { Injectable, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Erros de validacao
 *
 */
var errors = {
    required: 'Campo obrigatório',
    maxLength: 'Campo muito longo',
    minLength: 'Campo muito curto'
};
// End of file

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var CbForm = (function () {
    /**
     * Método constructor
     *
     * @param controllers
     */
    function CbForm(controllers, errorMessages) {
        /**
         * fBuilder
         *
         */
        this.builder = new FormBuilder();
        /**
         * Controllers do formulario
         *
         */
        this.controllers = {};
        /**
         * Mensagens de erro
         *
         */
        this.errorMessages = {};
        this.controllers = controllers;
        this.errorMessages = errorMessages;
    }
    /**
     * errors
     *
     * retorna os erros existentes em um campo
     *
     * @param {string} key nome do item do formulário
     */
    /**
     * errors
     *
     * retorna os erros existentes em um campo
     *
     * @param {?} key
     * @param {?=} customMessages
     * @return {?}
     */
    CbForm.prototype.errors = /**
     * errors
     *
     * retorna os erros existentes em um campo
     *
     * @param {?} key
     * @param {?=} customMessages
     * @return {?}
     */
    function (key, customMessages) {
        if (customMessages === void 0) { customMessages = {}; }
        // pega o controlador
        var /** @type {?} */ fControl = this.controllers[key];
        // se ele existir
        if (fControl) {
            // verifica se existem errors
            if (fControl.errors && fControl.invalid && fControl.dirty && fControl.touched) {
                // seta o array de retorno
                var /** @type {?} */ retorno = new Array();
                // percorre todos os erros
                for (var /** @type {?} */ e in fControl.errors) {
                    var /** @type {?} */ msg = (this.errorMessages[e]) ? this.errorMessages[e] : "Nenhuma mensagem encontrada para o erro " + e;
                    msg = (customMessages[e]) ? customMessages[e] : msg;
                    retorno.push(msg);
                }
                // volta as mensagens
                return retorno;
            }
            else
                return [];
        }
        else
            return [];
    };
    /**
     * control
     *
     * retorna um controller especifico do formulario
     *
     * @param {string} key nome do item do formulário
     */
    /**
     * control
     *
     * retorna um controller especifico do formulario
     *
     * @param {?} key
     * @return {?}
     */
    CbForm.prototype.control = /**
     * control
     *
     * retorna um controller especifico do formulario
     *
     * @param {?} key
     * @return {?}
     */
    function (key) {
        return (this.controllers[key]) ? this.controllers[key] : null;
    };
    /**
     * Obtem o form que devera ser colocado no HTML
     *
     */
    /**
     * Obtem o form que devera ser colocado no HTML
     *
     * @return {?}
     */
    CbForm.prototype.html = /**
     * Obtem o form que devera ser colocado no HTML
     *
     * @return {?}
     */
    function () {
        return this.builder.group(this.controllers);
    };
    return CbForm;
}());

// End of file

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var CbValidator = (function () {
    function CbValidator() {
        /**
         * fBuilder
         *
         */
        this.fBuilder = new FormBuilder();
        /**
         * Mensagens de erro
         *
         */
        this.errorMessages = errors;
        /**
         * Regras personalizadas
         *
         */
        this.customRules = {};
    }
    /**
     * Seta os controllers do form
     *
     * @param {?} rules
     * @return {?}
     */
    CbValidator.prototype.__parseControllers = /**
     * Seta os controllers do form
     *
     * @param {?} rules
     * @return {?}
     */
    function (rules) {
        // objeto que contera os forms controller
        var /** @type {?} */ formControlObject = {};
        // percorre todas as regras
        for (var /** @type {?} */ r in rules) {
            // pega o array de validadores
            var /** @type {?} */ validatorsArray = this.__getValidatorsArray(rules[r], r);
            // adiciona um novo FormControl
            formControlObject[r] = new FormControl(null, validatorsArray);
        }
        // retorna o objeto de formControl
        return formControlObject;
    };
    /**
     * Gera um novo formulário
     *
     * @param {?} rules
     * @return {?}
     */
    CbValidator.prototype.build = /**
     * Gera um novo formulário
     *
     * @param {?} rules
     * @return {?}
     */
    function (rules) {
        // Cria um novo formulário
        return new CbForm(this.__parseControllers(rules), this.errorMessages);
    };
    /**
     * Cria uma nova regra
     *
     * @param {?} rule
     * @return {?}
     */
    CbValidator.prototype.createRule = /**
     * Cria uma nova regra
     *
     * @param {?} rule
     * @return {?}
     */
    function (rule) {
        this.customRules[rule.name] = rule;
    };
    /**
     * __getValidatorsArray
     *
     * seta uma array de validadores customizados
     *
     * @author Gustavo Vilas Boas
     * \@since 02-2017
     * @param {?} rules
     * @param {?} field
     * @return {?}
     */
    CbValidator.prototype.__getValidatorsArray = /**
     * __getValidatorsArray
     *
     * seta uma array de validadores customizados
     *
     * @author Gustavo Vilas Boas
     * \@since 02-2017
     * @param {?} rules
     * @param {?} field
     * @return {?}
     */
    function (rules, field) {
        // verifica se existem regras
        if (!rules)
            return new Array();
        // separa as regras de validação
        var /** @type {?} */ rulesArray = rules.split('|');
        // declara o array de retorno
        var /** @type {?} */ retorno = [];
        // percorre todas as regras
        for (var /** @type {?} */ r in rulesArray) {
            // verifica se existe []
            if (rulesArray[r].indexOf('[') !== -1 && rulesArray[r].indexOf(']') !== -1) {
                // separa a string
                var /** @type {?} */ parts = rulesArray[r].split(/[[\]]{1,2}/);
                parts.length--;
                // Adiciona a regra
                var /** @type {?} */ toPush = this.customRules[parts[0]] ? this.customRules[parts[0]].validator(parts[1]) : Validators[parts[0]](parts[1]);
                retorno.push(toPush);
            }
            else {
                // Adiciona a regra
                var /** @type {?} */ toPush = this.customRules[rulesArray[r]] ? this.customRules[rulesArray[r]].validator : Validators[rulesArray[r]];
                retorno.push(toPush);
            }
        }
        // retorna as regras
        return retorno;
    };
    CbValidator.decorators = [
        { type: Injectable },
    ];
    return CbValidator;
}());

// End of file

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var Rule = (function () {
    /**
     * Método constructor
     *
     * @param name nome da regra
     * @param validator função de validação
     * @param label label da regra
     */
    function Rule(name, validator, label) {
        // Seta as propriedades do formulário
        this.name = name;
        this.validator = validator;
        this.label = label;
    }
    /**
     * Verifica se um determinado valor é válido para essa regra
     *
     * @param value
     */
    /**
     * Verifica se um determinado valor é válido para essa regra
     *
     * @param {?} value
     * @return {?}
     */
    Rule.prototype.valid = /**
     * Verifica se um determinado valor é válido para essa regra
     *
     * @param {?} value
     * @return {?}
     */
    function (value) {
        return this.validator(value);
    };
    return Rule;
}());

// End of file

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var CbFormModule = (function () {
    function CbFormModule() {
    }
    /**
     * @return {?}
     */
    CbFormModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: CbFormModule,
            providers: [CbValidator]
        };
    };
    CbFormModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule,
                        ReactiveFormsModule,
                        FormsModule
                    ],
                    declarations: [],
                    exports: [
                        ReactiveFormsModule,
                        FormsModule
                    ]
                },] },
    ];
    return CbFormModule;
}());

export { CbValidator, CbForm, Rule, CbFormModule };
